/// <reference types="node" />
import { StateManager } from "@nomicfoundation/ethereumjs-statemanager";
import { Account, Address } from "@nomicfoundation/ethereumjs-util";
import { JsonRpcClient } from "../../jsonrpc/client";
import { GenesisAccount } from "../node-types";
export declare const randomHashSeed: () => Buffer;
export declare class ForkStateManager implements StateManager {
    private readonly _jsonRpcClient;
    private readonly _forkBlockNumber;
    addresses: Set<string>;
    private _state;
    private _stateRoot;
    private _stateRootToState;
    private _originalStorageCache;
    private _stateCheckpoints;
    private _contextBlockNumber;
    private _contextChanged;
    constructor(_jsonRpcClient: JsonRpcClient, _forkBlockNumber: bigint, stateRoot?: string);
    static withGenesisAccounts(jsonRpcClient: JsonRpcClient, forkBlockNumber: bigint, genesisAccounts: GenesisAccount[]): Promise<ForkStateManager>;
    copy(): ForkStateManager;
    getAccount(address: Address): Promise<Account>;
    putAccount(address: Address, account: Account): Promise<void>;
    touchAccount(_address: Address): void;
    putContractCode(address: Address, value: Buffer): Promise<void>;
    getContractCode(address: Address): Promise<Buffer>;
    getContractStorage(address: Address, key: Buffer): Promise<Buffer>;
    putContractStorage(address: Address, key: Buffer, value: Buffer): Promise<void>;
    clearContractStorage(address: Address): Promise<void>;
    checkpoint(): Promise<void>;
    commit(): Promise<void>;
    revert(): Promise<void>;
    getStateRoot(): Promise<Buffer>;
    setStateRoot(stateRoot: Buffer): Promise<void>;
    dumpStorage(_address: Address): Promise<Record<string, string>>;
    hasGenesisState(): Promise<boolean>;
    generateCanonicalGenesis(): Promise<void>;
    generateGenesis(_initState: any): Promise<void>;
    accountIsEmpty(address: Address): Promise<boolean>;
    cleanupTouchedAccounts(): Promise<void>;
    setBlockContext(stateRoot: Buffer, blockNumber: bigint, irregularState?: Buffer): void;
    restoreForkBlockContext(stateRoot: Buffer): void;
    accountExists(_address: Address): never;
    deleteAccount(address: Address): Promise<void>;
    clearOriginalStorageCache(): void;
    getOriginalContractStorage(address: Address, key: Buffer): Promise<Buffer>;
    private _putAccount;
    private _setStateRoot;
    hasStateRoot(root: Buffer): Promise<boolean>;
    flush(): Promise<void>;
    modifyAccountFields(address: Address, accountFields: any): Promise<void>;
}
//# sourceMappingURL=ForkStateManager.d.ts.map